#include "handmade.h"

#define PI32 3.141592653589f

internal void renderPlayer(GameOffscreenBuffer *buffer, int playerX, int playerY) {
    uint8 *endOfBuffer = (uint8 *)buffer->memory + buffer->pitch*buffer->height;
    uint32 color = 0xFFFFFFFF;
    int top = playerY;
    int bottom = playerY + 10;
    for (int x = playerX; x < playerX + 10; x++) {
        uint8 *pixel = ((uint8 *)buffer->memory + x * buffer->bytesPerPixel + top*buffer->pitch);
        for (int y = top; y < bottom; y++) {
            if ((pixel >= buffer->memory) && (pixel + 4 < endOfBuffer)) {
                *(uint32 *)pixel = color;
            }
            pixel += buffer->pitch;
        }
    }

}

internal void outputSineSound(GameSoundOutputBuffer *soundBuffer, GameState *state) {
    int16 toneVolume = 3000;
    int wavePeriod = soundBuffer->samplesPerSecond/state->toneHz;

    int16 *sampleOut = soundBuffer->samples;
    for (int sampleIndex = 0; sampleIndex < soundBuffer->sampleCount; sampleIndex++) {
        state->tSine += 2.0f * PI32 / (real32)wavePeriod;
#if 0
        int16 sampleValue = (int16)(sin(state->tSine) * (real32)toneVolume);
#else
        int16 sampleValue = 0;
#endif
        *sampleOut++ = sampleValue;
        *sampleOut++ = sampleValue;
    }
}

internal void renderWeirdGradient(GameOffscreenBuffer *buffer, int xOffset, int yOffset) {
    uint8 *row = (uint8 *)buffer->memory;
    for (int y = 0; y < buffer->height; y++) {
        uint32 *pixel = (uint32*)row;
        for (int x = 0; x < buffer->width; x++) {
            uint8 blue = (uint8)(x + xOffset);
            uint8 green = (uint8)(y + yOffset);
            *pixel++ = (green << 8) | blue;
        }
        row += buffer->pitch;
    }
}

extern "C" GAME_UPDATE_AND_RENDER(gameUpdateAndRender) {
    Assert(sizeof(GameState) <= memory->permanentStorageSize);

    GameState *state = (GameState*)memory->permanentStorage;

    if (!memory->isInitialised) {
        DebugReadFileResult bmpMem = memory->debugReadEntireFile(__FILE__);
        if (bmpMem.contents) {
            if (false) {
                memory->debugWriteEntireFile("c:/source/repos/handmade/src/test.cpp", bmpMem.contentsSize, bmpMem.contents);
            }
            memory->debugFreeFileMemory(bmpMem.contents);
        }

        state->toneHz = 440;
        state->tSine = 0;
        state->playerY = 100;
        state->playerX = 100;
        state->blueOffset = 0;
        state->greenOffset = 0;
        memory->isInitialised = true;
    }

    for (int controllerIndex = 0; controllerIndex < ArrayCount(input->controllers); controllerIndex++) {
        GameControllerInput *controllerInput = &input->controllers[controllerIndex];
        /*
        if (controllerInput->isAnalog) {
            state->toneHz = 440 + (int)(128.0f*controllerInput->stickAvgX);
            state->greenOffset -= (int)(20.0f*controllerInput->stickAvgX);
            state->blueOffset += (int)(20.0f*controllerInput->stickAvgY);
        } else {
            if (controllerInput->stickRight.endedDown) {
                state->toneHz = 440 + 128;
                state->greenOffset -= 10;
            } else if (controllerInput->stickLeft.endedDown) {
                state->toneHz = 440 - 128;
                state->greenOffset += 10;
            }
            if (controllerInput->stickUp.endedDown) {
                state->blueOffset += 10;
            } else if (controllerInput->stickDown.endedDown) {
                state->blueOffset -= 10;
            }
        }
        */

        if (controllerInput->btnDown.endedDown) {
            state->playerY -= 30;
        }

        state->playerX += (int)(4.0f*controllerInput->stickAvgX);
        state->playerY -= (int)(4.0f*controllerInput->stickAvgY);
    }

    renderWeirdGradient(videoBuf, state->greenOffset, state->blueOffset);
    renderPlayer(videoBuf, state->playerX, state->playerY);
}

extern "C" GAME_GET_SOUND_SAMPLES(gameGetSoundSamples) {
    outputSineSound(soundBuf, (GameState*)memory->permanentStorage);
}
